# -------------------------------------------------------------------------------------------------
# Common
from PyQuantum.Common.Assert import *
from PyQuantum.Common.Print import *
from PyQuantum.Common.STR import *
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
class Cavity:

    # ---------------------------------------------------------------------------------------------
    def __init__(self, wc, wa, g, n_atoms, n_levels=2):
        Assert(isinstance(wc, (int, float)), "wc is not numeric", cf())
        Assert(isinstance(wa, (int, float)), "wa is not numeric", cf())
        Assert(isinstance(g, (int, float)), "g is not numeric", cf())
        Assert(isinstance(n_atoms, int), "n_atoms is not integer", cf())

        Assert(wc > 0, "wc <= 0", cf())
        Assert(wa > 0, "wa <= 0", cf())
        Assert(g > 0, "g <= 0", cf())
        Assert(n_atoms > 0, "n <= 0", cf())

        self.wc = wc
        self.wa = wa

        self.g = g

        self.n_atoms = n_atoms
        self.n_levels = n_levels

    # ---------------------------------------------------------------------------------------------

    # ---------------------------------------------------------------------------------------------

    def print_n_atoms(self):
        print(" n_atoms: ", color="yellow")

        print(self.n_atoms)

        print()
    # ---------------------------------------------------------------------------------------------

    # ---------------------------------------------------------------------------------------------
    def print_wc(self):
        print("wc: ", color="yellow")

        print(wc_str(self.wc))

        print()
    # ---------------------------------------------------------------------------------------------

    # ---------------------------------------------------------------------------------------------
    def print_wa(self):
        print("wa: ", color="yellow")

        print(wa_str(self.wa))

        print()
    # ---------------------------------------------------------------------------------------------

    # ---------------------------------------------------------------------------------------------
    def print_g(self):
        print(" g: ", color="yellow")

        print(g_str(self.g))

        print()
    # ---------------------------------------------------------------------------------------------

    # ---------------------------------------------------------------------------------------------
    def print(self, title="Cavity:"):
        print(title, color="green")

        print()

        self.print_wc()
        self.print_wa()
        self.print_g()
        self.print_n_atoms()
    # ---------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
